from ....Classes.DataKeeper import DataKeeper


def get_force_datakeeper(self, symbol_list):
    """
    Generate DataKeepers to store by default results from force module

    Parameters
    ----------
    self: VarLoad
        A VarLoad object
    symbol_list : list
        List of the existing datakeeper (to avoid duplicate)

    Returns
    -------
    dk_list: list
        list of DataKeeper
    """
    dk_list = []

    return dk_list
