from ....Methods.Slot.Slot import SlotCheckError


class S11_W01CheckError(SlotCheckError):
    """ """

    pass


class S11_RWCheckError(SlotCheckError):
    """ """

    pass


class S11_RHCheckError(SlotCheckError):
    """ """

    pass


class S11_H1rCheckError(SlotCheckError):
    """ """

    pass
