class PointArc1Error(Exception):
    """ """

    pass


class RadiusArc1Error(Exception):
    """ """

    pass


class NbPointArc1DError(Exception):
    """ """

    pass


class AngleRotationArc1Error(Exception):
    """ """

    pass


class PointTranslateArc1Error(Exception):
    """ """

    pass
