from ....Methods.Slot.Slot import SlotCheckError


class S61_InnerCheckError(SlotCheckError):
    """ """

    pass


class S61_W21CheckError(SlotCheckError):
    """ """

    pass


class S61_RCheckError(SlotCheckError):
    """ """

    pass


class S61_WindHError(SlotCheckError):
    """ """

    pass


class S61_WindWError(SlotCheckError):
    """ """

    pass


class S61_WindError(SlotCheckError):
    """

    Parameters
    ----------

    Returns
    -------

    Raises
    ------
    winding


    """

    pass
