from ....Methods import NotImplementedYetError


def comp_force_nodal(self, output):
    """Compute the nodal air-gap surface force based on Maxwell Tensor (MT).

    Parameters
    ----------
    self : ForceMT
        A ForceMT object

    output : Output
        an Output object (to update)
    """

    raise NotImplementedYetError("Nodal force not available yet for Maxwell tensor")
