from ....Methods.Slot.Slot import SlotCheckError


class S52_NoneError(SlotCheckError):
    """Raised when a propery of HoleM52 is None"""

    pass


class S52_H12CheckError(SlotCheckError):
    """ """

    pass


class S52_alphaCheckError(SlotCheckError):
    """ """

    pass


class S52_W1CheckError(SlotCheckError):
    """ """

    pass
