# -*- coding: utf-8 -*-


def check(self):
    """Check that the Slot object is correct

    Parameters
    ----------
    self : Slot
        A Slot object

    Returns
    -------
    None
    """
    pass
