# -*- coding: utf-8 -*-


def is_synchronous(self):
    """Return if a machine is synchronous or not

    Parameters
    ----------
    self : MachineWRSM
        A MachineWRSM

    Returns
    -------
    is_synchronous: bool
        WRSM machines are synchronous

    """
    return True
