# -*- coding: utf-8 -*-


def get_begin(self):
    """Return the begin point of the arc

    Parameters
    ----------
    self : Arc2
        An Arc2 object

    Returns
    -------
    begin: complex
        begin point of the Arc2
    """

    return self.begin
