def pop(self, i):
    """
    Remove the item at the given position in the list, and return it.
    If no index is specified, a.pop() removes and returns the last item in the list.
    """
    return self.list_output.pop(i)
