from ....Methods.Slot.Slot import SlotCheckError


class S51_NoneError(SlotCheckError):
    """Raised when a propery of HoleM51 is None"""

    pass


class S51_WCheckError(SlotCheckError):
    """ """

    pass


class S51_RHCheckError(SlotCheckError):
    """ """

    pass


class S51_AlphaCheckError(SlotCheckError):
    """ """

    pass
