from ....Methods.Slot.Slot import SlotCheckError


class S27_W01CheckError(SlotCheckError):
    """ """

    pass


class S27_W12CheckError(SlotCheckError):
    """ """

    pass


class S27_W03CheckError(SlotCheckError):
    """ """

    pass
