# -*- coding: utf-8 -*-


def get_end(self):
    """Return the end point of the arc

    Parameters
    ----------
    self : Arc3
        An Arc3 object

    Returns
    -------
    end: complex
        Complex coordinates of the end point of the Arc3
    """

    return self.end
