# -*- coding: utf-8 -*-


def get_begin(self):
    """Return the begin point of the segment

    Parameters
    ----------
    self : Segment
        A Segment object

    Returns
    -------
    begin: complex
        Begin point of the Segment
    """

    return self.begin
