from ....Methods.Slot.Slot import SlotCheckError


class S60_InnerCheckError(SlotCheckError):
    """ """

    pass


class S60_W21CheckError(SlotCheckError):
    """ """

    pass


class S60_RCheckError(SlotCheckError):
    """ """

    pass


class S60_WindHError(SlotCheckError):
    """ """

    pass


class S60_WindWError(SlotCheckError):
    """ """

    pass


class S60_WindError(SlotCheckError):
    """

    Parameters
    ----------

    Returns
    -------

    Raises
    ------
    winding


    """

    pass
