from ....Methods.Slot.Slot import SlotCheckError


class S54_NoneError(SlotCheckError):
    """Raised when a propery of HoleM54 is None"""

    pass


class H54_W0CheckError(SlotCheckError):
    """ """

    pass


class H54_R1CheckError(SlotCheckError):
    """ """

    pass
