class VentilationTrapInstanceError(Exception):
    """Raised when a Ventilation Trap parameter is not int or float"""

    pass


class TrapezeBuildGeometryError(Exception):
    """raised when the parameter have not good type"""

    pass
