from ....Methods.Slot.Slot import SlotCheckError


class S10_W01CheckError(SlotCheckError):
    """ """

    pass


class S10_W02CheckError(SlotCheckError):
    """ """

    pass


class S10_H1rCheckError(SlotCheckError):
    """ """

    pass
