# -*- coding: utf-8 -*-


def get_name_phase(self):
    """Return the name of the winding phases

    Parameters
    ----------
    self : Lamination
        A Lamination object

    Returns
    -------
    name_phase: list
        Empty list

    """
    return []
