# -*- coding: utf-8 -*-
from ....Methods.Slot.Slot import SlotCheckError


class S28_WCheckError(SlotCheckError):
    """ """

    pass


class S28_RboW0CheckError(SlotCheckError):
    """ """

    pass


class S28_R1W3CheckError(SlotCheckError):
    """ """

    pass


class S28_R1R1CheckError(SlotCheckError):
    """ """

    pass


class S28_ZsCheckError(SlotCheckError):
    """ """

    pass
