# -*- coding: utf-8 -*-


def check(self):
    """Check that the Conductor object is correct

    Parameters
    ----------
    self : Conductor
        A Conductor object

    Returns
    -------
    None
    """
    pass
