# -*- coding: utf-8 -*-

import numpy as np


def jacobian(self, point, nodes):
    """Compute jacobian, jacobian determinant and jacobian derivatives for linear triangle.

    Parameters
    ----------
    :param self : a RefElement object

    Returns
    -------

    """
    # TODO
    return None, None
