# -*- coding: utf-8 -*-

pixmap_dict = {
    "End_Random": ":/images/images/MachineSetup/WindParam/End_Random.PNG",
    "End_Preform": ":/images/images/MachineSetup/WindParam/End_Preform.PNG",
    "Dim_Ext_Rotor": ":/images/images/MachineSetup/MachineDimension/Dimension_Stator_Rotor.png",
    "Dim_In_Rotor_No_Shaft": ":/images/images/MachineSetup/MachineDimension/Dimension_Rotor_Stator.png",
    "Dim_In_Rotor_Shaft": ":/images/images/MachineSetup/MachineDimension/Dimension_Shaft_Rotor_Stator.png",
    "SCIM": ":/images/images/MachineSetup/MachineType/machine_SCIM.png",
    "DFIM": ":/images/images/MachineSetup/MachineType/DFIM.png",
    "SynRM": ":/images/images/MachineSetup/MachineType/machine_SynRM.png",
    "SPMSM": ":/images/images/MachineSetup/MachineType/machine_SPMSM.png",
    "IPMSM": ":/images/images/MachineSetup/MachineType/machine_IPMSM.png",
    "WRSM": ":/images/images/MachineSetup/MachineType/WRSM.png",
    "cross": ":/images/images/icon/cross.png",
    "check": ":/images/images/icon/check.png",
    "search": ":/images/images/icon/search.png",
}
