from ....Methods.Machine.LamSlotWind import Lam_WindCheckError


class SquirrelCageCheckError(Lam_WindCheckError):
    """ """

    pass


class SCC_NotARotor(SquirrelCageCheckError):
    """ """

    pass


class SCC_WrongCond(SquirrelCageCheckError):
    """ """

    pass
