# -*- coding: utf-8 -*-


def remove_magnet(self):
    """Remove the magnet (set to None) of the Hole

    Parameters
    ----------
    self : HoleM53
        a HoleM53 object
    """

    self.magnet_0 = None
    self.magnet_1 = None
