# Default number of point added for the discrtization of an arc
ARC_NPOINT_D = 99
# Default number of point added for the discrtization of a line
LINE_NPOINT_D = 0


class MachineCheckError(Exception):
    """ """

    pass
