# -*- coding: utf-8 -*-


def get_center(self):
    """Return the center point of the arc

    Parameters
    ----------
    self : Arc2
        An Arc2 object

    Returns
    -------
    center: complex
        center point of the Arc2
    """

    return self.center
