def comp_length_endwinding(self):
    """Compute the Winding overhang length on one side for a half-turn[m]

    Parameters
    ----------
    self: Winding
        A Winding object
    Returns
    -------
    Lewout : float
        End-winding length on one side for a half-turn [m].
    """

    # TODO: compute it
    return self.Lewout
