from ....Methods.Slot.Slot import SlotCheckError


class S13_W01CheckError(SlotCheckError):
    """ """

    pass


class S13_W12CheckError(SlotCheckError):
    """ """

    pass


class S13_H1rCheckError(SlotCheckError):
    """ """

    pass
