def insert(self, i, x):
    """
    Insert an item at a given position.
    The first argument is the index of the element before which to insert,
    so a.insert(0, x) inserts at the front of the list, and a.insert(len(a), x) is equivalent to a.append(x).
    """
    return self.output_list.insert(i, x)
