# -*- coding: utf-8 -*-
from numpy import dot


def is_inside(self, vertice, point, normal_t=None):
    """Check if a point is inside the cell defined by the vertices.

    Parameters
    ----------
    self : RefTriangle3
        an RefTriangle3 object
    vertice : ndarray
        vertices of the cell
    point : ndarray
        coordinates of the checked point
    normal_t : ndarray
        (optional) cell normal vector

    Returns
    -------
    is_inside: bool
        true if the point is inside the cell

    """
    # TODO
    return None
