class PointArc3Error(Exception):
    """ """

    pass


class NbPointArc3DError(Exception):
    """ """

    pass


class AngleRotationArc3Error(Exception):
    """ """

    pass


class PointTranslateArc3Error(Exception):
    """ """

    pass
