class PointArc2Error(Exception):
    """ """

    pass


class AngleArc2Error(Exception):
    """ """

    pass


class NbPointArc2DError(Exception):
    """ """

    pass


class AngleRotationArc2Error(Exception):
    """ """

    pass


class PointTranslateArc2Error(Exception):
    """ """

    pass
