def comp_angle_opening(self):
    """Compute the average opening angle of the Slot

    Parameters
    ----------
    self : SlotW16
        A SlotW16 object

    Returns
    -------
    alpha: float
        Average opening angle of the slot [rad]

    """

    return self.W0
