from ....Methods.Machine import MachineCheckError


class LaminationCheckError(MachineCheckError):
    """ """

    pass


class LC_RadiusError(LaminationCheckError):
    """  """

    pass


class LC_VentYokeOut(LaminationCheckError):
    """ """

    pass


class LC_VentYokeIn(LaminationCheckError):
    """ """

    pass
