class PointSegmentError(Exception):
    """ """

    pass


class AngleRotationSegmentError(Exception):
    """ """

    pass


class PointTranslateSegmentError(Exception):
    """ """

    pass


class NbPointSegmentDError(Exception):
    """ """

    pass
