# -*- coding: utf-8 -*-


def has_magnet(self):
    """Return if the Hole has magnets

    Parameters
    ----------
    self : Hole
        A Hole object

    Returns
    -------
    has_magnet : bool
        False

    """

    return False
