from ....Methods.Slot.Slot import SlotCheckError


class S58_NoneError(SlotCheckError):
    """Raised when a propery of HoleM58 is None"""

    pass


class S58_WCheckError(SlotCheckError):
    """ """

    pass
