# -*- coding: utf-8 -*-


def load_columns(self):
    """Method to load the names of the columns data of the Elmer simulation results

    Parameter
    ---------


    Return
    ------

    """
    pass
