# -*- coding: utf-8 -*-

from ....Methods.Machine.Winding import WindingError


class WindingT1DefMsError(WindingError):
    """

    Parameters
    ----------

    Returns
    -------

    Raises
    ------
    must
        be 0

    """

    pass
