from ....Methods.Machine.Lamination import LaminationCheckError


class Lam_WindCheckError(LaminationCheckError):
    """ """

    pass


class LWC_OutSlotInLam(Lam_WindCheckError):
    """ """

    pass


class LWC_InSlotOutLam(Lam_WindCheckError):
    """ """

    pass


class LWC_SlotTooLong(Lam_WindCheckError):
    """ """

    pass


class LWC_OverlappingSlot(Lam_WindCheckError):
    """ """

    pass
