# -*- coding: utf-8 -*-

import numpy as np


def shape_function(self, points):
    """ Return the values of linear shape functions in reference triangle for a given point """
    # TODO
    return None, None
