from ....Methods.Slot.Slot import SlotCheckError


class S50_NoneError(SlotCheckError):
    """Raised when a propery of HoleM50 is None"""

    pass


class S50_W01CheckError(SlotCheckError):
    """ """

    pass


class S50_H23CheckError(SlotCheckError):
    """ """

    pass


class S50_H01CheckError(SlotCheckError):
    """ """

    pass


class S50_W5CheckError(SlotCheckError):
    """ """

    pass


class S50_SpCheckError(SlotCheckError):
    """ """

    pass
