# -*- coding: utf-8 -*-


def has_magnet(self):
    """Return if the Hole has magnet

    Parameters
    ----------
    self : HoleMag
        A HoleMag object

    Returns
    -------
    has_magnet : bool
        True

    """

    return True
